<?php

namespace Drupal\views_numeric_range_filter\Plugin\views\filter;

use Drupal\views\Plugin\views\filter\NumericFilter;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Database\Query\Condition;

/**
 * Simple filter to handle greater than/less than filters.
 *
 * @ingroup views_filter_handlers
 *
 * @ViewsFilter("numeric_range_filter")
 */
class NumericRangeFilter extends NumericFilter {

  public function hasExtraOptions() {
    if ($this->operator == 'range') {
      return TRUE;
    }
  }

  protected function defineOptions() {
    $options = parent::defineOptions();

    $options['range_min'] = ['default' => ''];
    $options['range_max'] = ['default' => ''];
    $options['range_operator'] = ['default' => 'integer'];

    return $options;
  }

  public function operators() {
    $operators = parent::operators();
    $operators += [
        'range' => [
          'title' => $this->t('Numeric range'),
          'method' => 'op_range',
          'short' => $this->t('Range'),
          'values' => 1,
        ],
      ];
    return $operators;
  }

  /**
   * Provide a list of all the numeric operators.
   */
  public function buildExtraOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildExtraOptionsForm($form, $form_state);
    if ($this->operator == 'range') {
      $form['range_min'] = [
        '#type' => 'number',
        '#title' => $this->t('- Minus'),
        '#default_value' => $this->options['range_min'],
        '#description' => $this->t('Starting value from the exposed filter minus this amount becomes the minimum search value.'),
      ];
      $form['range_max'] = [
        '#type' => 'number',
        '#title' => $this->t('+ Plus'),
        '#default_value' => $this->options['range_max'],
        '#description' => $this->t('Starting value from the exposed filter plus this amount becomes the maximum search value.'),
      ];
      $form['range_operator'] = [
        '#type' => 'select',
        '#title' => $this->t('Operator'),
        '#options' => [
          'integer' => $this->t('Integer'),
          'percent' => $this->t('Percent'),
          'multiple' => $this->t('Multiple'),
        ],
        '#default_value' => $this->options['range_operator'],
        '#description' => $this->t('The operator to use'),
      ];
    }
  }
  
  /**
   * Views Numeric Rang Filter
   */
  function op_range($field) {

    $operator = $this->options['range_operator'];
    $minus = $this->options['range_min'];
    $plus = $this->options['range_max'];
    $value = $this->value['value'];

    switch ($operator) {
      case 'percent' :
        if(isset($value) && is_numeric($value)) {
          $min = $value - (($minus * 0.01) * $value);
          $max = (($plus * 0.01) * $value) + $value;
          break;
        }
        elseif($value == '' || !is_numeric($value)) {
          $value = 0;
          $min = $value - (($minus * 0.01) * $value);
          $max = (($plus * 0.01) * $value) + $value;
          break;
        }
      case 'multiple' :
        if(isset($value) && is_numeric($value)) {
          $min = $value - ($value * $minus);
          $max = ($value * $plus) + $value;
          break;    
        }
        elseif($value == '' || !is_numeric($value)) {
          $value = 0;
          $min = $value - ($value * $minus);
          $max = ($value * $plus) + $value;
          break;
        }
      default:
      case 'integer' :
        if(isset($value) && is_numeric($value)) {
          $min = $value - $minus;
          $max = $value + $plus;
          break;
        }
        elseif($value == '' || !is_numeric($value)) {
          $value = 0;
          $min = $value - $minus;
          $max = $value + $plus;
          break;
        }
      }    
    if ($this->operator == 'range') {
      $this->query->addWhere($this->options['group'], $field, [$min, $max], 'BETWEEN');
    }
    else {
      $or = new Condition('OR');
      $this->query->addWhere($this->options['group'],
      $or
        ->condition($field, $min, '<=')
        ->condition($field, $max, '>=')
      );
    }
  }
}
